/*
 * MainBoard_UART.c
*************************************************************************
	Author: Alexander von Lhmann
	
	Hardware
   	processor:		AtMega644A
	clock:			32.768khz external RC oscillator

	Function		Pin		Word	Connected With

	Comm. Module 1
	TX1_M1			PA6				PD3 (RX1_M1)
	TX2_M1			PA7				PD2 (RX2_M1)
	RX_M1			PA5				PD1 (TX_M1)
	
	Comm. Module 2
	TX1_M2			PA3				PD3 (RX1_M2)
	TX2_M2			PA4				PD2 (RX2_M2)
	RX_M2			PA2				PD1 (TX_M2)
	
	Comm. Module 3
	TX1_M3			PA0				PD3 (RX1_M3)
	TX2_M3			PA1				PD2 (RX2_M3)
	RX_M3			PB0				PD1 (TX_M3)
	
	Comm. Module 4
	TX1_M4			PB2				PD3 (RX1_M4)
	TX2_M4			PB1				PD2 (RX2_M4)
	RX_M4			PB3				PD1 (TX_M4)

	UART:
	RXD0			PD0				TX_BT				
	TXD0			PD1				RX_BT
	RTS				PD2				RTS#BT
	CTS				PD3				CTS#BT
	
	SPI:
	MOSI			PB5				SDI ADC
	MISO			PB6				SDO ADC
	SS				PB4				+5V				
	SCK				PB7				SCK ADC
	Slave Select	PC0				CS ADC
	
	LED1			PC4				not running
	LED2			PC6			
	

*************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include "stdbool.h"
#include <string.h>

#define F_CPU 20E6 //Systemtakt in Hz
#define UART_BAUD_RATE 38400  

#include <util/delay.h>

#include "SPI_functions.h"
#include "uart.h"
#include "LTC2486_functions.h"
#include "Mainboard_UART_functions.h"



// Global VARIABLES

unsigned char byte_buffer;
uint16_t timer_ms_counter;								// x*10ms-Time-Value	


void LED2_on() {
	PORTC |= (1<<PINC6);
}

void LED2_off() {
	PORTC &= ~(1<<PINC6);
}

// Timer Compare Match A Interrupt (each ~10ms)
ISR(TIMER1_COMPA_vect) {
	timer_ms_counter++;
}


int main(void) {
	
	/* Variables */
	char controlbyte;
	bool speedmode = false;
	bool tempmeas = false;
	uint8_t active_channel= 0;
	uint8_t active_module= 1;
	volatile uint16_t adc_config =0;							// ADC-Config Data (for LTC2468)
	volatile uint16_t adc_value =0;							// Extracted ADC-Value
	volatile uint32_t adc_receive =0;						// Received ADC Value
	volatile uint16_t temp_value =0;						//Received and extracted Temp-Value
	
	unsigned int c;
	char buffer[7];
	int  num=134;
			
	/*********************** Initialization ***********************/
	//Port Directions
	
	DDRA= 0b11011011;				// NIRS-Module 1-3 Communication									
	DDRB= 0b10100110;				// NIRS-Module 3-4 Communication, SPI										
	DDRC= 0b01010001;				// SPI Slave select, LEDs										
	DDRD= 0b00001010;				// UART
	
	//Initializations
	uart_init(UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU));					// UART initialization
	SPI_init();					// SPI initialization
	Timer_init();					// Timer initialization

	adc_config = LTC2486_create_config(active_module,speedmode,tempmeas,1);
	
	sei();							// Global Interrupt enable
	
    while(1) {	
			
        c = uart_getc();				//  Get received character from ringbuffer: lower byte = received character, higher byte (bitmask) = last receive error
        if ( c & UART_NO_DATA )			// no data available from UART 
        {
			adc_receive = LTC2486_SPI_get_value(adc_config);							// No Data available
			adc_value = LTC2486_extract_value(adc_receive);
			send_ADC_data(active_module, active_channel, speedmode, adc_value, timer_ms_counter);
			_delay_ms(500);
        }
        else
        {
            /*
             * check for Frame or Overrun error
             */
            if ( c & UART_FRAME_ERROR )							// Framing Error detected, i.e no stop bit detected 
            {
                uart_puts_P("UART Frame Error: ");
            }
            if ( c & UART_OVERRUN_ERROR )						// Overrun, a character already present in the UART UDR register was not read by the interrupt handler before the next character arrived, one or more received characters have been dropped
            {
                uart_puts_P("UART Overrun Error: ");
            }
            if ( c & UART_BUFFER_OVERFLOW )						// We are not reading the receive buffer fast enough, one or more received character have been dropped 
            {
                uart_puts_P("Buffer overflow error: ");
            }
			
            /* 
             * send received character back
             */
            uart_putc( (unsigned char)c );
			
			
        }
		
    }
}